package ru.pricepulse.parsingservice.config;

import jakarta.annotation.PostConstruct;
import org.springframework.core.io.Resource;
import org.springframework.core.io.ResourceLoader;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

import static ru.pricepulse.parsingservice.wildberries_parser.proxy.ProxyChecker.checkProxies;
import static ru.pricepulse.parsingservice.wildberries_parser.proxy.ProxyChecker.readProxiesFromFile;
import static ru.pricepulse.parsingservice.wildberries_parser.proxy.ProxyChecker.saveProxiesToFile;

@Component
public class ProxyProvider {
    private List<String> workingProxies;

    private final AtomicInteger currentProxyIndex = new AtomicInteger(0);

    private final ResourceLoader resourceLoader;

    public ProxyProvider(ResourceLoader resourceLoader) {
        this.resourceLoader = resourceLoader;
    }

    //@PostConstruct
    public void init() throws IOException {
        Resource proxy = resourceLoader.getResource("classpath:proxy.txt");
        Resource okProxy = resourceLoader.getResource("classpath:ok-proxy.txt");
        List<String> proxies = Files.readAllLines(Path.of(proxy.getURI()));
        System.out.println("Начата проверка проксей");
        workingProxies = checkProxies(proxies);
        System.out.println("Закончена проверка проксей");


        saveProxiesToFile(workingProxies, Path.of(okProxy.getURI()));

        if (workingProxies.isEmpty()) {
            throw new RuntimeException("Нет доступных рабочих прокси.");
        }

        System.out.println("Найдено рабочих прокси: " + workingProxies.size());
    }

    public synchronized InetSocketAddress getNextProxy() {
        // Получаем текущий индекс прокси
        int currentIndex = currentProxyIndex.getAndUpdate(index -> (index + 1) % workingProxies.size());

        String[] proxy = workingProxies.get(currentIndex).split(":");
        return new InetSocketAddress(proxy[0], Integer.parseInt(proxy[1]));
    }
}
