package ru.pricepulse.parsingservice.wildberries_parser.proxy;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.InetSocketAddress;
import java.net.Proxy;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.stream.Collectors;

public class ProxyChecker {
    private static final int TIMEOUT = 2000;
    private static final int THREAD_COUNT = 30;

    public static List<String> readProxiesFromFile(String filePath) {
        try {
            return Files.readAllLines(Paths.get(filePath));
        } catch (IOException e) {
            System.err.println("Ошибка при чтении файла: " + e.getMessage());
            return new ArrayList<>();
        }
    }

    public static List<String> checkProxies(List<String> proxies) {
        ExecutorService executor = Executors.newFixedThreadPool(THREAD_COUNT);
        List<Future<String>> futures = new ArrayList<>();

        // Отправляем задачи проверки прокси в пул потоков
        for (String proxyAddress : proxies) {
            futures.add(executor.submit(() -> isProxyWorking(proxyAddress) ? proxyAddress : null));
        }

        // Получаем результаты выполнения
        List<String> workingProxies = futures.stream()
                .map(future -> {
                    try {
                        return future.get();
                    } catch (Exception e) {
                        System.err.println("Ошибка при получении результата проверки прокси: " + e.getMessage());
                        return null;
                    }
                })
                .filter(proxy -> proxy != null)
                .collect(Collectors.toList());

        executor.shutdown(); // Завершаем работу пула потоков
        return workingProxies;
    }

    private static boolean isProxyWorking(String proxyAddress) {
        String[] parts = proxyAddress.split(":");
        if (parts.length != 2) {
            System.err.println("Некорректный формат прокси: " + proxyAddress);
            return false;
        }

        String ip = parts[0];
        int port;

        try {
            port = Integer.parseInt(parts[1]);
        } catch (NumberFormatException e) {
            System.err.println("Некорректный порт у прокси: " + proxyAddress);
            return false;
        }

        try {
            Proxy proxy = new Proxy(Proxy.Type.HTTP, new InetSocketAddress(ip, port));
            URL url = new URL("http://www.google.com");
            HttpURLConnection connection = (HttpURLConnection) url.openConnection(proxy);
            connection.setConnectTimeout(TIMEOUT);
            connection.setReadTimeout(TIMEOUT);
            connection.setRequestMethod("GET");
            connection.connect();

            int responseCode = connection.getResponseCode();
            if (
                    responseCode == 200
//                    responseCode == 403 ||
//                    responseCode == 500 ||
//                    responseCode == 407 ||
//                    responseCode == 501
            ) {
                System.out.println("Прокси работает (код ответа " + responseCode + "): " + proxyAddress);
                return true;
            } else {
                System.out.println("Прокси не отвечает (код ответа " + responseCode + "): " + proxyAddress);
                return false;
            }
        } catch (IOException e) {
            System.out.println("Прокси не отвечает: " + proxyAddress);
            return false;
        }
    }

    public static void saveProxiesToFile(List<String> proxies, Path filePath) {
        try (BufferedWriter writer = Files.newBufferedWriter(filePath)) {
            for (String proxy : proxies) {
                writer.write(proxy);
                writer.newLine();
            }
        } catch (IOException e) {
            System.err.println("Ошибка при записи в файл: " + e.getMessage());
        }
    }
}
